local P = Apollo.GetAddon("PieMenu")

local ktMenu = P.PieMenu:new({
  strId = "PvP",
  strName = "PvP Menu",
  oKeybind = P.PieKey:new("ctrl-alt-p"),
  fnPopulateItems = function()
    local tRet = {}
    -- if not in a battleground, queue options
    if not MatchingGame.IsInMatchingGame() and not MatchingGame.IsInPVPGame() then
      local bIsQueued = MatchingGame.IsQueuedForMatching()
      if bIsQueued then
        table.insert(tRet, { strName = "Leave Queue", strIcon = "ClientSprites:LootCloseBox", func = function()
            if MatchingGame.CanLeaveQueueAsGroup() then
              MatchingGame.LeaveMatchingQueueAsGroup()
            elseif 	MatchingGame.IsQueuedForMatching() then
              MatchingGame.LeaveMatchingQueue()
            end
          end
        })
        return tRet
      end
      
      local tGames = MatchingGame.GetAvailableMatchingGames(MatchingGame.MatchType.Battleground)
      for i, matchGame in ipairs(tGames) do
        if matchGame:GetName() == "Random" then
          table.insert(tRet, {
            strName = matchGame:GetName(),
            strIcon = "ClientSprites:GroupRandomLootIcon",
            func = function() MatchingGame.Queue({ [matchGame:GetName()] = matchGame }) end,
          })
        elseif matchGame:GetName() == "Walatiki Temple" then
          table.insert(tRet, {
            strName = matchGame:GetName(),
            strIcon = "ClientSprites:Icon_BuffBattlegrounds_Mask_Carrier",
            func = function() MatchingGame.Queue({ [matchGame:GetName()] = matchGame }) end,
          })
        elseif matchGame:GetName() == "Halls of the Bloodsworn" then
          table.insert(tRet, {
            strName = matchGame:GetName(),
            strIcon = "ClientSprites:Icon_SkillPhysical_FountainOfBlood",
            func = function() MatchingGame.Queue({ [matchGame:GetName()] = matchGame }) end,
          })
        else
          table.insert(tRet, {
            strName = matchGame:GetName(),
            strIcon = "ClientSprites:Icon_Windows_UI_CRB_LevelUp_NewPvPBattleground",
            func = function() MatchingGame.Queue({ [matchGame:GetName()] = matchGame }) end,
          })
        end
      end
      
      return tRet
    end
    
  	local tMatchState = MatchingGame:GetPVPMatchState()
    if not tMatchState then
      return tRet
    end
    
    if tMatchState.eState == MatchingGame.PVPGameState.Finished then
      table.insert(tRet, { strName = "Leave Match", strIcon = "ClientSprites:Icon_Mission_Explorer_ExplorerDoor", func = function() if MatchingGame.IsInMatchingGame() then MatchingGame.LeaveMatchingGame() end end })
      return
    end
    
    for key, peCurrent in pairs(PublicEvent.GetActiveEvents()) do
      local eType = peCurrent:GetEventType()
      if eType == PublicEvent.PublicEventType_PVP_Battleground_Vortex then            -- Walatiki Temple
        table.insert(tRet, { strName = "Attack their Mask Carrier", strIcon = "ClientSprites:Icon_BuffBattlegrounds_Mask_Carrier", strInput = "/instance --- Attack their Mask Carrier ---" })
        table.insert(tRet, { strName = "Defend our Mask Carrier", strIcon = "ClientSprites:Icon_ItemArmorFace_UI_Item_Battle_Face", strInput = "/instance --- Defend our Mask Carrier ---" })
        table.insert(tRet, { strName = "Steal their Masks", strIcon = "ClientSprites:Icon_ItemWeaponPowerglove_UI_Item_Powerfist_001", strInput = "/instance --- Steal their Masks ---" })
        table.insert(tRet, { strName = "Enemy Incoming", strIcon = "ClientSprites:Icon_BuffBattlegrounds_Power_Up_Damage", strInput = "/instance --- Enemy Incoming ---" })
        table.insert(tRet, { strName = "Enemy Spotted", strIcon = "ClientSprites:Icon_ItemMisc_UI_Item_Lopp", strInput = "/instance --- Enemy Spotted ---" })
        table.insert(tRet, { strName = "Capture the Unearthed Mask", strIcon = "ClientSprites:Icon_ItemMisc_TorineInsignia", strInput = "/instance --- Capture the Unearthed Mask ---" })
        table.insert(tRet, { strName = "Defend our Capture Point", strIcon = "ClientSprites:Icon_Mission_Soldier_HoldoutProtect", strInput = "/instance --- Defend our Capture Point ---" })
        table.insert(tRet, { strName = "Attack their Capture Point", strIcon = "ClientSprites:Icon_Mission_Soldier_HoldoutConquer", strInput = "/instance --- Attack their Capture Point ---" })
        break
      elseif eType == PublicEvent.PublicEventType_PVP_Battleground_HoldTheLine then   -- Halls of the Bloodsworn
        table.insert(tRet, { strName = "Attack Main Point", strIcon = "ClientSprites:Icon_Mission_Explorer_ClaimTerritory_TEMP_Red", strInput = "/instance --- Attack Main Point ---" })
        table.insert(tRet, { strName = "Attack Right Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshrt", strInput = "/instance --- Attack Right Capture Point ---" })
        table.insert(tRet, { strName = "Capture Right Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshrt", strInput = "/instance --- Capture Right Capture Point ---" })
        table.insert(tRet, { strName = "Defend Right Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshrt", strInput = "/instance --- Defend Right Capture Point ---" })
        table.insert(tRet, { strName = "Defend Main Point", strIcon = "ClientSprites:Icon_Mission_Explorer_ClaimTerritory", strInput = "/instance --- Defend Main Point ---" })
        table.insert(tRet, { strName = "Enemy Incoming", strIcon = "ClientSprites:Icon_BuffBattlegrounds_Power_Up_Damage", strInput = "/instance --- Enemy Incoming ---" })
        table.insert(tRet, { strName = "Enemy Spotted", strIcon = "ClientSprites:Icon_ItemMisc_UI_Item_Lopp", strInput = "/instance --- Enemy Spotted ---" })
        table.insert(tRet, { strName = "Teleport to Next Stage!", strIcon = "ClientSprites:Icon_SkillPetCommand_Combat_Pet_Stop_and_Return", strInput = "/instance --- Teleport to Next Stage! ---" })
        table.insert(tRet, { strName = "Defend Left Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshlft", strInput = "/instance --- Defend Left Capture Point ---" })
        table.insert(tRet, { strName = "Capture Left Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshlft", strInput = "/instance --- Capture Left Capture Point ---" })
        table.insert(tRet, { strName = "Attack Left Capture Point", strIcon = "ClientSprites:Icon_SkillMisc_UI_misc_dshlft", strInput = "/instance --- Attack Left Capture Point ---" })
				break
      elseif eType == PublicEvent.PublicEventType_PVP_Warplot then                    -- Warplot!
        break
      else
        -- TODO: Other types
      end
    end    
    
    return tRet
  end,
})

P:RegisterMenu(ktMenu)