local P = Apollo.GetAddon("PieMenu")
local DaiGUI = Apollo.GetPackage("DaiGUI-1.0").tPackage

local function OnChangeKeybind(_, wndHandler, wndControl)
  if wndHandler ~= wndControl then return end
  local tMenu = wndControl:GetParent():GetData()
  
  P.wndOptions:FindChild("KeybindBlocker"):Show(true)
  P.wndOptions:FindChild("KeybindBlocker:KeybindBlockerMiddleBG:Text"):SetText(tMenu.strName)
  P.bKeyBindMode = true
  P.tKeyBindMenu = tMenu
  P.wndKeyBind = wndControl
  
  for idx, wndChild in ipairs(P.wndOptions:FindChild("KeybindContainer"):GetChildren()) do
    wndChild:FindChild("BindButton"):Enable(false)
  end
  
  wndControl:SetCheck(true)
  wndControl:SetFocus()
end

local function PopulateKeybinds(_, wndHandler, wndControl)
  if wndHandler ~= wndControl then return end
  local wndContainer = wndControl:FindChild("KeybindContainer")
  for i, tMenu in ipairs(P.tMenus) do
    DaiGUI:Create({ 
      LuaData = tMenu, 
      AnchorPoints = "HFILL", 
      AnchorOffsets = { 0, 0, -10, 30 }, 
      Picture = true,
      IgnoreMouse = true,
      Pixies = {
        { Text = tMenu.strName, DT_VCENTER = true, AnchorPoints = "FILL", AnchorOffsets = { 9, 0, -185, 0 }, TextColor = "UI_WindowTextDefault", Font = "CRB_InterfaceMedium" },
      },
      Children = {
        {
          Class = "Button",
          ButtonType = "PushButton",
          Name = "BindButton",
          AnchorPoints = {1,0,1,1},
          AnchorOffsets = { -186, 3, -1, -3 },
          Font = "CRB_InterfaceMedium",
          ProcessRightClick = true,
          DT_CENTER = true,
          DT_VCENTER = true,
          Base = "CRB_Basekit:kitBtn_List_Holo",
          TextTheme = "UI_BtnTextBlue",
          Text = tostring(tMenu.oKeybind or ""),
          Events = {
            ButtonSignal = OnChangeKeybind,
          },
        },
      },
    }):GetInstance(nil, wndContainer)
  end
  wndContainer:ArrangeChildrenVert(0)
end

function P:ShowOptionsWindow()
  if self.wndOptions and self.wndOptions:IsValid() then 
    self.wndOptions:Destroy()
  end
  
  local tWndDef = {
    Name = "PieMenuOptionsWindow",
    AnchorCenter = { 600, 420 },
    Escapable = true,
    Moveable = true,
    Children = {
      {
        Class            = "Button",
        Name             = "DisableInputCascade",
        AnchorOffsets    = {65, 65, 415, 95 },
        RelativeToClient = true,
        Font             = "CRB_InterfaceMedium",
        ButtonType       = "Check",
        TextColor        = "UI_WindowTextDefault",
        DrawAsCheckbox   = true,
        DT_VCENTER       = true,
        TextTheme        = "UI_BtnTextBlue",
        Base             = "BK3:btnHolo_Check",
        Text             = "  Disable Modifier Cascading (Recommended)",
        Tooltip          = "When modifier cascading is enabled any keybinds pressed will also trigger keybinds that partially match what is pressed.\n\nFor example: Pressing the CTRL-ALT-Q keybind will also trigger the Q keybind.",
        Events = {
          ButtonCheck = function(_, wndHandler, wndControl)
            if wndHandler ~= wndControl then return end
            Apollo.SetConsoleVariable("input.cascadeModifiers", not wndControl:IsChecked())
          end,
          ButtonUncheck = "Event::ButtonCheck",
          WindowLoad = function(_, wndHandler, wndControl)
            if wndHandler ~= wndControl then return end
            wndControl:SetCheck(not Apollo.GetConsoleVariable("input.cascadeModifiers"))
          end,
        },
      },
      { -- CloseButton
        Name = "CloseButton",
        Class = "Button",
        ButtonType = "PushButton",
        Base = "BK3:btnHolo_Close", 
        WindowSoundTemplate = "CloseWindowPhys",
        AnchorPoints = "TOPRIGHT", AnchorOffsets = { -65, 31, -34, 62 },
        NoClip = true,
        Events = {
          ButtonSignal = function(_, wndHandler, wndControl) wndControl:GetParent():Close() end,
        },
      },
      {
        Text = "PieMenu keybinds will NOT replace existing game keybinds.\nALT as a modifier will only work when CTRL is held down with it.",
        Font = "CRB_InterfaceSmall",
        TextColor = "xkcdPaleRed",
        AnchorPoints  = "HFILL",
        AnchorOffsets = { 70, 100, -70, 140 },
        DT_VCENTER = true,
        DT_WORDBREAK = true,
      },
      {
        Name = "KeybindBlocker",
        AnchorPoints = "FILL",
        Sprite = "BlackFill",
        BGColor = "9dffffff",
        AnchorOffsets = { 60, 150, -60, -60 },
        NewWindowDepth = true,
        TransitionShowHide = true,
        Visible = false,
        SwallowMouseClicks = true,
        Overlapped = true,
        Children = {
          {
            Name = "KeybindBlockerMiddleBG",
            AnchorPoints = "CENTER",
            AnchorOffsets = { -125, -58, 125, 57 },
            Sprite = "CRB_Basekit:kitBase_HoloBlue_Small",
            Children = {
              {
                Name = "Text",
                AnchorPoints = "HFILL",
                AnchorOffsets = {16, 38, -16, 58},
                TextColor = "UI_BtnTextRedNormal",
                Font = "CRB_InterfaceSmall_O",
                DT_CENTER = true,
                DT_WORDBREAK = true,
                Text = "",
              },
              {
                WidgetType = "PushButton",
                Name = "UnbindButton",
                Text = "Unbind",
                AnchorPoints = { 0,0,0.5,1 },
                AnchorOffsets = {16,64,-8,-16},
                BGColor = "ffff0000",
                Font = "CRB_InterfaceSmall_O",
                DT_CENTER = true,
                DT_VCENTER = true,
                Base = "CRB_Basekit:kitBtn_Holo_Red",
                TextTheme = "UI_BtnTextRed",
                SwallowMouseClicks = true,
                Overlapped = true,
                Events = {
                  ButtonSignal = function(_, wndHandler, wndControl)
                    if wndHandler ~= wndControl then return end
                    if P.tKeyBindMenu == nil then return end
                    P.tKeyBindMenu.oKeybind = nil
                    P.wndKeyBind:SetText("")
                    P:StopKeyBindMode()
                  end,
                },
              },
              {
                WidgetType = "PushButton",
                Name = "CancelButton",
                Text = "Cancel",
                AnchorPoints = { 0.5, 0, 1, 1 },
                AnchorOffsets = { 8, 64, -16, -16 },
                Font = "CRB_InterfaceSmall_O",
                DT_CENTER = true,
                DT_VCENTER = true,
                Base = "CRB_Basekit:kitBtn_Holo",
                TextTheme = "UI_BtnTextHolo",
                SwallowMouseClicks = true,
                Overlapped = true,
                Events = {
                  ButtonSignal = function(_, wndHandler, wndControl)
                    if wndHandler ~= wndControl then return end
                    if P.tKeyBindMenu == nil then return end
                    P:StopKeyBindMode()
                  end,
                },
              },
            },
          },
        },
      },
      {
        Name = "KeybindContainer",
        AnchorPoints = "FILL",
        VScroll = true,
        --AutoHideScroll = true,
        Template = "CRB_Scroll_HoloLarge",
        AnchorOffsets = { 60, 150, -60, -60 },
      },
    },
    Pixies = {
      { AnchorPoints = "HFILL", AnchorOffsets = { 0, 40, 0, 60 }, DT_VCENTER = true, DT_CENTER = true, Text = "PieMenu Options", Font = "CRB_InterfaceLarge_BO", TextColor = "UI_TextHoloTitle" },
      { AnchorPoints = "FILL", AnchorOffsets = { 50, 140, -50, -50 }, Sprite = "CRB_Basekit:kitInnerFrame_Holo_SimpleInset" },
      { AnchorFill   = true, Sprite = "BK3:UI_BK3_Holo_Framing_2" }, -- CRB_Basekit:kitOuterFrame_MetalGray_Medium" },
    },
    Events = {
      WindowLoad = PopulateKeybinds,
      WindowClosed = function()
        P:StopKeyBindMode()
      end,
    }
  }
  self.wndOptions = DaiGUI:Create(tWndDef):GetInstance()
end
