local P       = Apollo.GetAddon("PieMenu")
local DaiUtil = Apollo.GetPackage("DaiUtil-1.0").tPackage

P.PieKey = {}
local PieKey = P.PieKey

local setmetatable, type, pairs, ipairs = setmetatable, type, pairs, ipairs

local ktKeyTranslation = {
  [8]   = "BACKSPACE",
  [9]   = "TAB",
  [13]  = "ENTER",
  [19]  = "PAUSE",
  [20]  = "CAPSLOCK",
  [27]  = "ESCAPE",
  [32]  = "SPACE",
  [33]  = "PAGE UP",
  [34]  = "PAGE DOWN",
  [35]  = "END",
  [36]  = "HOME",
  [37]  = "LEFT",
  [38]  = "UP",
  [39]  = "RIGHT",
  [40]  = "DOWN",
  [45]  = "INSERT",
  [46]  = "DELETE",
  [96]  = "NUM 0",
  [97]  = "NUM 1",
  [98]  = "NUM 2",
  [99]  = "NUM 3",
  [100] = "NUM 4",
  [101] = "NUM 5",
  [102] = "NUM 6",
  [103] = "NUM 7",
  [104] = "NUM 8",
  [105] = "NUM 9",
  [106] = "NUM *",
  [107] = "NUM +",
  [109] = "NUM -",
  [110] = "NUM .",
  [111] = "NUM /",
  [112] = "F1",
  [113] = "F2",
  [114] = "F3",
  [115] = "F4",
  [116] = "F5",
  [117] = "F6",
  [118] = "F7",
  [119] = "F8",
  [120] = "F9",
  [121] = "F10",
  [122] = "F11",
  [123] = "F12",
  [144] = "NUMLOCK",
  [186] = ";",
  [187] = "+",
  [188] = ",",
  [189] = "-",
  [190] = ".",
  [191] = "/",
  [192] = "`",
  [219] = "[",
  [220] = "\\",
  [221] = "]",
  [222] = "'",  
}




-- PieKey tostring
function PieKey.tostring(s)
  return (bit32.band(s.eModifier, GameLib.CodeEnumInputModifier.Shift) > 0 and "SHIFT-" or "") ..
         (bit32.band(s.eModifier, GameLib.CodeEnumInputModifier.Control) > 0 and "CTRL-" or "") ..
         (bit32.band(s.eModifier, GameLib.CodeEnumInputModifier.Alt) > 0 and "ALT-" or "") ..
         (ktKeyTranslation[s.nKey] and ktKeyTranslation[s.nKey] or  string.char(s.nKey))
end

function PieKey.eq(a, b)
  return a.nKey == b.nKey and a.eModifier == b.eModifier
end

-- PieKey constructor
function PieKey:new(arg)
  -- nothing passed to the constructor... return nil
  if arg == nil then 
    return 
  end
  
  local o = {
    nKey = 0,
    eModifier = 0,
  }
  
  setmetatable(o, self)
  self.__index = self
  self.__tostring = PieKey.tostring  
  self.__eq = PieKey.eq

  -- parse the argument
  local t
  if type(arg) == "number" then
    t = { nKey = arg, eModifier = Apollo.GetMetaKeysDown() }
  elseif type(arg) == "string" then
    t = PieKey.Parse(arg)
  elseif type(arg) == "table" then
    t = arg
  end
  
  -- merge the parsed data
  if t ~= nil then
    for k,v in pairs(t) do
      o[k] = t[k] or o[k]
    end
  end
  
  return o
end

-- Compare PieKey to individual components
function PieKey:Is(nKey, eModifier)
  return nKey == self.nKey and
         eModifier == self.eModifier
end

-- Parse a string-based keybind
-- TODO: Requires handling of non-alphanumeric keys such as F1-F12, Home, End, etc
function PieKey.Parse(s)
  -- return nil if keybind is empty
  if s == nil or s:len() == 0 then
    return
  end
    
  local tParts = DaiUtil.StringSplit(s:upper(), "-")
  local tKey = {
    nKey = 0,
    eModifier = 0,
  }
  for _, p in ipairs(tParts) do
    if p == "SHIFT" then
      tKey.eModifier = bit32.bor(tKey.eModifier, GameLib.CodeEnumInputModifier.Shift)
   elseif p == "CTRL" then
      tKey.eModifier = bit32.bor(tKey.eModifier, GameLib.CodeEnumInputModifier.Control)
    elseif p == "ALT" then
      tKey.eModifier = bit32.bor(tKey.eModifier, GameLib.CodeEnumInputModifier.Alt)
    else
      for nKey, sName in pairs(ktKeyTranslation) do
        if sName == p or sName:gsub(" ", "") == p then
          tKey.nKey = nKey
          break
        end
      end
      if tKey.nKey == 0 then
        tKey.nKey = string.byte(p)
      end
    end
  end
  
  if tKey.nKey ~= nil then
    return tKey
  end
  
  -- return nil if no key was found
end